/*
 * @Author: wuhao930406 1148547900@qq.com
 * @Date: 2023-08-03 11:31:00
 * @LastEditors: wuhao930406 1148547900@qq.com
 * @LastEditTime: 2023-08-24 14:27:24
 * @FilePath: /editor-plus/src/pages/components/GlftBlock/index.jsx
 * @Description:
 *
 * Copyright (c) 2023 by ${git_name_email}, All Rights Reserved.
 */
import { Gltf, OrbitControls, Stage, useGLTF } from "@react-three/drei";
import { Canvas, useThree } from "@react-three/fiber";
import { useEffect, useRef, useState } from "react";
import * as THREE from "three";
import "./index.less";
import { useTimeout } from "ahooks";

function Sphere({ position, CoordinatePoint, CurGongjianPoint }) {
  const meshRef = useRef();

  return (
    <mesh
      ref={meshRef}
      scale={100}
      position={
        position
          ? Object.values(position)?.map((it, i) => {
              const offset = Object.values(CoordinatePoint);
              const offset1 = Object.values(CurGongjianPoint);
              let res =
                parseFloat(it * 10000000) +
                parseFloat(offset[i] * 10000000000) -
                parseFloat(offset1[i] * 10000000000);
              res = res / 10000000000;
              const diyoffset = i === 0 ? 0.021 : i === 1 ? 0.012 : 0.005;
              return (res + diyoffset) * 100;
            })
          : []
      }
    >
      <sphereGeometry attach="geometry" args={[0.006, 32, 32]} />
      <meshPhysicalMaterial attach="material" color="red" />
    </mesh>
  );
}

const Model = ({ url }) => {
  const { camera } = useThree();
  const cameraRef = useRef(camera);
  const { scene } = useGLTF(url);
  const [position, setPosition] = useState([0, 0, 0]);

  useEffect(() => {
    // 计算模型的包围盒
    const box = new THREE.Box3().setFromObject(scene);
    const center = new THREE.Vector3();
    box.getCenter(center);
    let position = [-center.x * 240, -center.y * 240, -center.z * 240];
    setPosition(position);
    cameraRef.current.position.set(8, 8, 16); // 设置摄像机位置为 (5, 5, 5)
    cameraRef.current.lookAt(0, 0, 0); // 使摄像机指向原点
  }, [scene]);

  return (
    <Gltf
      src={url}
      receiveShadow
      castShadow
      position={position}
      scale={240}
      inject={
        <meshPhysicalMaterial
          color="white"
          metalness={1} // 控制金属感，1 表示完全金属
          roughness={0} // 控制光滑度，0 表示非常光滑
        />
      }
    />
  );
};

function PointViewer({
  position,
  CoordinatePoint,
  CurGongjianPoint,
  CurGongjianData,
}) {
  console.log("====================================");
  console.log(CurGongjianData);
  console.log("====================================");

  const [url, seturl] = useState(null);

  useTimeout(()=>{
    seturl(`./glb/${CurGongjianData.ModelPath}.glb`)
  },100)


  return (
    <div className="bar-block-component">
      {CurGongjianPoint && (
        <Canvas
          shadows
          dpr={[1, 1.5]}
          gl={{ antialias: true }}
          camera={{ position: [8, 8, 16], fov: 75 }}
          style={{ height: "100%", width: "100%" }}
          id={"rrt"}
        >
          <color attach="background" args={["#333"]} />
          <Stage
            intensity={0.5}
            preset="rembrandt"
            shadows={"contact"}
            adjustCamera={1}
            environment={{
              path: "/",
              files: "env.hdr",
            }}
          >
            <axesHelper args={[20]} position={[0, 0, 0]}></axesHelper>
            {url && (
              <Model url={url}></Model>
            )}
            <Sphere
              position={position}
              CoordinatePoint={CoordinatePoint}
              CurGongjianPoint={CurGongjianPoint}
            ></Sphere>
          </Stage>
          <OrbitControls></OrbitControls>
        </Canvas>
      )}
    </div>
  );
}

export default PointViewer;
